#include "myIncludes.h"
#include "fred.h"

#if NeedFunctionProtoTypes
static void exposeAlive(FredClass _fred, GC _theGC);
static void exposeDead(FredClass _fred, GC _theGC);
static void draw(FredClass _fred);
static void unDraw(FredClass _fred);
static void move(FredClass _fred);
static void realMove(FredClass _fred);
static void demoMove(FredClass _fred);
static int destroy(FredClass _fred);
static void initialize(FredClass _fred);
static void reset(FredClass _fred);
#else
static void exposeAlive();
static void exposeDead();
static void draw();
static void unDraw();
static void move();
static void realMove();
static void demoMove();
static int destroy();
static void initialize();
static void reset();
#endif


#define DEAD_CLOCK 30 

FredClassRec fred = 
{
  {
    FredObject,
    NULL,
    FALSE,
    sizeof(FredClassRec),
    initialize, 
    draw,
    unDraw,
    demoMove,
    destroy,
    reset,
    NULL,
    0.25,
    Yellow,
    (GC)NULL,
    0.0,
    0.0,
    Still,
    Gonzales,
    Alive,
    4,
  },
  {
    0,
    0,
    4,
  },
};

ObjectClass Fred = (ObjectClass)&fred;

static void initialize(FredClass _fred)
{
  SETCOLOR(_fred->object.gc, _fred->object.color);
}

static void exposeAlive(FredClass _fred, GC _theGC)
{
  int angle1, angle2;
  int space;

  space = 30 * _fred->fred.mouth;

  switch(_fred->object.direction)
  {
    case Up:
      angle1 = (135-space/2)*64;
      angle2 = (270+space)*64;
      break;
    case Down:
      angle1 = (315-space/2)*64;
      angle2 = (270+space)*64;
      break;
    case Left:
      angle1 = (225-space/2)*64;
      angle2 = (270+space)*64;
      break;
    case Still:
    case Right:
      angle1 = (45-space/2)*64;
      angle2 = (270+space)*64;
      break;
  }

  XFillArc(TheDisplay, TheWindow, _theGC, (int)XCOORD(_fred->object.x)+1,
    (int)YCOORD(_fred->object.y)+1, TheBoxWidth-2, TheBoxHeight-2, 
    angle1, angle2);

}

static void exposeDead(FredClass _fred, GC _theGC)
{
  int angle1, angle2;
  double percent;

  angle1 = 120*64*_fred->fred.mouth;
  angle2 = 270*64;

  percent = (double)_fred->object.clock/(double)DEAD_CLOCK;

  XFillArc(TheDisplay, TheWindow, _theGC, XCOORD(_fred->object.x)+1+TheBoxWidth/2*(1.0-percent),
    YCOORD(_fred->object.y)+1+TheBoxHeight/2*(1.0-percent), 
    (TheBoxWidth-2)*percent, (TheBoxHeight-2)*percent,
    angle1, angle2);
}

static void draw(FredClass _fred)
{
  if (_fred->object.state == Alive)
    exposeAlive(_fred, _fred->object.gc);
  else
    exposeDead(_fred, _fred->object.gc);
}

static void unDraw(FredClass _fred)
{
  if (_fred->object.state == Alive)
    exposeAlive(_fred, TheBackgroundGC);
  else
    exposeDead(_fred, TheBackgroundGC);
}

static void realMove(FredClass _fred)
{
  int current;
  int up, left, right, down;
  double x, y;
  int try = 0;

  x = _fred->object.x;
  y = _fred->object.y;

  _fred->fred.mouth = (_fred->fred.mouth == 3) ? 0 : _fred->fred.mouth + 1;

  if (_fred->object.state == Dead)
  {
    if (_fred->object.clock)
      _fred->object.clock--;
    else
    {
      Reset(&GameList);
      _fred->object.state = Alive;
      _fred->object.direction = Still;
    }
    return;
  }
  move(_fred);
}

static void demoMove(FredClass _fred)
{

  if (_fred->object.direction == Still)
    _fred->object.direction = Left;

  if (Random() > 3)
    _fred->object.direction = MoveRandom((ObjectClass)_fred);
  else
    _fred->object.direction = MoveToChar((ObjectClass)_fred, DOT_CHAR);

  _fred->fred.mouth = (_fred->fred.mouth == 3) ? 0 : _fred->fred.mouth + 1;

  if (_fred->object.state == Dead)
  {
    if (_fred->object.clock)
      _fred->object.clock--;
    else
    {
      Reset(&GameList);
      _fred->object.state = Alive;
      _fred->object.direction = Left;
    }
    return;
  }

  move(_fred);
}

static void move(FredClass _fred)
{
  switch(_fred->object.direction)
  {
    case Still:
      return;
    case Up:
      if (GoodUp((ObjectClass)_fred))
        _fred->object.y -= _fred->object.moveInc;
      else
        _fred->object.y = ROUND(_fred->object.y);
      _fred->object.x = ROUND(_fred->object.x);
      break;
    case Down:
      if (GoodDown((ObjectClass)_fred))
        _fred->object.y += _fred->object.moveInc;
      else
        _fred->object.y = ROUND(_fred->object.y);
      _fred->object.x = ROUND(_fred->object.x);
      break;
    case Left:
      if (GoodLeft((ObjectClass)_fred))
        _fred->object.x -= _fred->object.moveInc;
      else
        _fred->object.x = ROUND(_fred->object.x);
      _fred->object.y = ROUND(_fred->object.y); 
      break;
    case Right:
      if (GoodRight((ObjectClass)_fred))
        _fred->object.x += _fred->object.moveInc;
      else
        _fred->object.x = ROUND(_fred->object.x);
      _fred->object.y = ROUND(_fred->object.y);
      break;
  }

  if (RemoveMazeElement(_fred->object.x, _fred->object.y) == POWER_CHAR)
  {
    int i;
    for(i=0;i<NUM_GHOSTS;i++)
      if (TheGhost[i]->object.state == Alive)
        ScareGhost((GhostClass)TheGhost[i]);
  }
}

static int destroy(FredClass _fred)
{
  int i;

  _fred->fred.men--;

  if (_fred->fred.men == 0)
    GameOver();

  _fred->fred.mouth = 0;
  _fred->object.clock = DEAD_CLOCK;
  _fred->object.state = Dead;
}

static void reset(FredClass _fred)
{
  char *ptr;

  DrawMazeCharUnderObject((ObjectClass)_fred); 
  ptr = strchr(TheMaze, FRED_CHAR);
  if (ptr == (char *)NULL)
    ptr = strchr(TheMaze, PATH_CHAR);
 
  MoveObject(TheFred, XPOS(ptr-TheMaze), YPOS(ptr-TheMaze));

  _fred->object.clock = 0;
  _fred->fred.mouth = 0;
  _fred->object.state = Alive;
  _fred->object.direction = Still;
}

/*
** External functions.
*/

void FredSetDemo()
{
  reset(&fred);
  fred.object.move = demoMove;
  fred.object.direction = Left;
}

void FredSetReal()
{
  reset(&fred);
  fred.object.move = realMove;
  fred.object.direction = Still;
}
