#include "myIncludes.h"
#include "fruit.h"

#define FRUIT32DIR "fruit32"
#define FRUIT16DIR "fruit16"

#define DEFAULTDIR "."

#if NeedFunctionProtoTypes
static void expose(FruitClass _fruit, GC _theGC);
static void postFunc(FruitClass _fruit);
static void classInit();
static void initialize(FruitClass _fruit);
static int destroy(FruitClass _fruit);
static void reset(FruitClass _fruit);
static void move(FruitClass _fruit);
static void draw(FruitClass _fruit);
static void unDraw(FruitClass _fruit);
#else
static void expose();
static void postFunc();
static void classInit();
static void initialize();
static int destroy();
static void reset();
static void move();
static void draw();
static void unDraw();
#endif

FruitClassRec fruit = 
{
  {
    FruitObject,
    classInit,
    FALSE,
    sizeof(FruitClassRec),
    initialize,
    draw,
    unDraw,
    move,
    destroy,
    reset,
    postFunc,
    0.0,
    White,
    (GC)NULL,
    0.0,
    0.0,
    Still,
    (Speed)NULL,
    Dead,
    5000,
  },
  {
    1000, 
    100,
    Cherry,
    (Pixmap)NULL,
    0,
    0,
  }
};

ObjectClass Fruit = (ObjectClass)&fruit;

static void expose(FruitClass _fruit, GC _theGC)
{
  int x, y;
  x = XCOORD(ROUND(_fruit->object.x));
  y = YCOORD(ROUND(_fruit->object.y));

  XCopyPlane(TheDisplay, _fruit->fruit.pixmap, TheWindow, _theGC, 1, 1,
    _fruit->fruit.pixWidth-2, _fruit->fruit.pixHeight-2, x, y, 1);
}

static void unDraw(FruitClass _fruit)
{
  expose(_fruit, TheBackgroundGC);
  DrawMazeCharUnderObject((ObjectClass)_fruit);
}

static void move(FruitClass _fruit)
{
/* I don't move. */
}

static void draw(FruitClass _fruit)
{
  if (_fruit->object.state == Alive)
    expose(_fruit, _fruit->object.gc);
}

static void classInit()
{
  char *ptr; 
  ptr = strchr(TheMaze, FRUIT_CHAR);
  if (ptr == (char *)NULL)
    ptr = strchr(TheMaze, PATH_CHAR);
 
  fruit.object.x = XPOS(ptr-TheMaze);
  fruit.object.y = YPOS(ptr-TheMaze);

  *ptr = DOT_CHAR;
}

static int destroy(FruitClass _fruit)
{
  _fruit->object.state = Dead;
  _fruit->object.clock = 500 * TheLevel/4;
  unDraw(_fruit);
  return _fruit->fruit.value * TheLevel;
}

static void initialize(FruitClass _fruit)
{
  char file[100];
  int status;
  int hot;
  int x, y;
  char *dir;

  x = ROUND(_fruit->object.x);
  y = ROUND(_fruit->object.y);

  *(TheMaze+x+y*MAZE_WIDTH) = DOT_CHAR;
   
  fruit.object.color = (fruit.object.color == (LastColor-1) ? Red : fruit.object.color + 1);
  _fruit->object.color = fruit.object.color;

  SETCOLOR(_fruit->object.gc, fruit.object.color);

  if (fruit.fruit.pixmap != (Pixmap)NULL)
    XFreePixmap(TheDisplay, fruit.fruit.pixmap);

  dir = (char *)getenv("XFREDDIR");

  if (dir == (char *)NULL)
    dir = DEFAULTDIR;

  if (TheBoxWidth == 32)
    sprintf(file, "%s/%s/fruit%d.xbm", dir, FRUIT32DIR, _fruit->fruit.type);
  else 
    sprintf(file, "%s/%s/fruit%d.xbm", dir, FRUIT16DIR, _fruit->fruit.type);

  status = XReadBitmapFile(TheDisplay, TheWindow, file,
    &fruit.fruit.pixWidth, &fruit.fruit.pixHeight, &fruit.fruit.pixmap,
    &hot, &hot);

  if ((status == BitmapOpenFailed) || (status == BitmapFileInvalid))
  {
    fruit.fruit.type = _fruit->fruit.type = 0;
    if (TheBoxWidth == 32)
      sprintf(file, "%s/%s/fruit%d.xbm", dir, FRUIT32DIR, _fruit->fruit.type);
    else 
      sprintf(file, "%s/%s/fruit%d.xbm", dir, FRUIT16DIR, _fruit->fruit.type);
    status = XReadBitmapFile(TheDisplay, TheWindow, file,
      &fruit.fruit.pixWidth, &fruit.fruit.pixHeight, &fruit.fruit.pixmap,
      &hot, &hot);

    if ((status == BitmapOpenFailed) || (status == BitmapFileInvalid))
    {
      printf("Fruit files are screwed up!\n");
      printf("Check environment variable XFREDDIR!\n");
      ShutDown(TheDisplay);
    }
  }

  _fruit->fruit.pixmap = fruit.fruit.pixmap;
  _fruit->fruit.pixWidth = fruit.fruit.pixWidth;
  _fruit->fruit.pixHeight = fruit.fruit.pixHeight;
  MoveObject((ObjectClass)_fruit, fruit.object.x, fruit.object.y);

}

static void reset(FruitClass _fruit)
{
  _fruit->object.state = Dead;
  _fruit->object.clock = 500 * TheLevel/4;
}

static void postFunc(FruitClass _fruit)
{
  if (_fruit->object.state == Dead)
  {
    _fruit->object.clock--;
    if (_fruit->object.clock <= 0)
    {
      _fruit->object.state = Alive;
      _fruit->fruit.timeLeft = 1000;
    }
  }
  else
  {
    _fruit->fruit.timeLeft--;
    if (_fruit->fruit.timeLeft <= 0)
    {
      _fruit->object.state = Dead;
      _fruit->object.clock = 500 * TheLevel/4;
    }
  }
}
