#include "myIncludes.h"
#include "ghost.h"

#if NeedFunctionProtoTypes
static void exposeAlive(GhostClass _ghost, GC _theGC);
static void exposeAfraid(GhostClass _ghost, GC _theGC);
static void exposeEyes(GhostClass _ghost, GC _theGC); 
static void draw (GhostClass _ghost);
static void unDraw(GhostClass _ghost);
static int destroy(GhostClass _ghost);
static void move();
static void classInit();
static void initialize(GhostClass _ghost);
static void reset(GhostClass _ghost);
static void checkAfraid();
#else
static void exposeAlive();
static void exposeAfraid();
static void exposeEyes();
static void draw();
static void unDraw();
static int destroy();
static void move();
static void classInit();
static void initialize();
static void reset();
static void checkAfraid();
#endif

GhostClassRec ghost = 
{
  {
    GhostObject,
    classInit,
    FALSE,
    sizeof(GhostClassRec),
    initialize,
    draw,
    unDraw,
    move,
    destroy,
    reset,
    NULL,
    0.25,
    White,
    (GC)NULL,
    0.0, 
    0.0,
    Up,
    Turtle,
    Alive,
    0,
  },
  {
    100,
    Stupid,
    Turtle,
    0,
    0,
    0,
  },
};

ObjectClass Ghost = (ObjectClass)&ghost;

static void initialize(GhostClass _ghost)
{
  Speed temp;

  ghost.object.color = (ghost.object.color == (LastColor-1) ) ? Red : ghost.object.color + 1;
  _ghost->object.color = ghost.object.color;

  _ghost->ghost.iQ = MIN(ROUND((TheLevel/2.0) * (double)Random()/10.0), LastIntelligence-1);

  _ghost->object.direction = Up;

  temp = (Random() % LastSpeed) + TheLevel/4;

  if (TheLevel < 15)
    _ghost->ghost.oldSpeed = _ghost->object.speed = MIN((LastSpeed-1), temp);
  else
    _ghost->ghost.oldSpeed = _ghost->object.speed = Warp9;
}

static void exposeAlive(GhostClass _ghost, GC _theGC)
{
  XFillArc(TheDisplay, TheWindow, _theGC, XCOORD(_ghost->object.x)+2,
    YCOORD(_ghost->object.y)+2, TheBoxWidth-4, TheBoxHeight-4,
    0, 180*64);

  XFillRectangle(TheDisplay, TheWindow, _theGC, XCOORD(_ghost->object.x),
    YCOORD(_ghost->object.y) + TheBoxWidth/2, TheBoxWidth, TheBoxHeight/2);
}

static void exposeAfraid(GhostClass _ghost, GC _theGC)
{
  XDrawArc(TheDisplay, TheWindow, _theGC, XCOORD(_ghost->object.x)+2,
    YCOORD(_ghost->object.y)+2, TheBoxWidth-4, TheBoxHeight-4,
    0, 180*64);
 
  XDrawRectangle(TheDisplay, TheWindow, _theGC, XCOORD(_ghost->object.x),
    YCOORD(_ghost->object.y) + TheBoxWidth/2, TheBoxWidth, TheBoxHeight/2);

}

static void exposeEyes(GhostClass _ghost, GC _theGC)
{
  XArc arcs[2];

  arcs[0].x = XCOORD(_ghost->object.x)+2;
  arcs[0].y = arcs[1].y = YCOORD(_ghost->object.y)+TheBoxHeight/4+2;
  arcs[1].x = XCOORD(_ghost->object.x)+TheBoxWidth/2 + 2;

  arcs[0].width = arcs[1].width = TheBoxWidth/2 - 4;
  arcs[0].height = arcs[1].height = TheBoxHeight/2 - 4;

  arcs[0].angle1 = arcs[1].angle1 = 0;
  arcs[0].angle2 = arcs[1].angle2 = 360*64;
  
  XFillArcs(TheDisplay, TheWindow, _theGC, 
    arcs, 2);

  XDrawArcs(TheDisplay, TheWindow, TheBackgroundGC,
    arcs, 2);

  switch(_ghost->object.direction)
  {
    case Up:
      arcs[0].x += arcs[0].width/4;
      arcs[0].y++;
      arcs[0].width /= 2;
      arcs[0].height /= 2;
      arcs[1].x += arcs[1].width/4;
      arcs[1].y++;
      arcs[1].width /= 2;
      arcs[1].height /= 2;
      XFillArcs(TheDisplay, TheWindow, TheBackgroundGC,
        arcs, 2);
      break;
    case Down:
      arcs[0].x += arcs[0].width/4;
      arcs[0].y += arcs[0].height/2;
      arcs[0].width /= 2;
      arcs[0].height /= 2;
      arcs[1].x += arcs[1].width/4;
      arcs[1].y += arcs[1].height/2;
      arcs[1].width /= 2;
      arcs[1].height /= 2;
      XFillArcs(TheDisplay, TheWindow, TheBackgroundGC,
        arcs, 2);
      break;
    case Left:
      arcs[0].x++;
      arcs[0].y += arcs[0].height/4;
      arcs[0].width /= 2;
      arcs[0].height /= 2;
      arcs[1].x++;
      arcs[1].y += arcs[1].height/4;
      arcs[1].width /= 2;
      arcs[1].height /= 2;
      XFillArcs(TheDisplay, TheWindow, TheBackgroundGC,
        arcs, 2);
      break;
    case Right:
      arcs[0].x += arcs[0].width/2;
      arcs[0].y += arcs[0].height/4;
      arcs[0].width /= 2;
      arcs[0].height /= 2;
      arcs[1].x += arcs[1].width/2;
      arcs[1].y += arcs[1].height/4;
      arcs[1].width /= 2;
      arcs[1].height /= 2;
      XFillArcs(TheDisplay, TheWindow, TheBackgroundGC,
        arcs, 2);
      break;
    case Still:
      arcs[0].x += arcs[0].width/4;
      arcs[0].y += arcs[0].height/4;
      arcs[0].width /= 2;
      arcs[0].height /= 2;
      arcs[1].x += arcs[1].width/4;
      arcs[1].y += arcs[1].height/4;
      arcs[1].width /= 2;
      arcs[1].height /= 2;
      XFillArcs(TheDisplay, TheWindow, TheBackgroundGC,
        arcs, 2);
      break;
  } 
}

static void draw(GhostClass _ghost)
{
  SETCOLOR(_ghost->object.gc, _ghost->object.color);

  switch(_ghost->object.state)
  {
    case Alive: 
gotoAlive:
      exposeAlive(_ghost, _ghost->object.gc);
      break;
    case Afraid:
gotoAfraid:
      XSetFillStyle(TheDisplay, _ghost->object.gc, FillOpaqueStippled);
      exposeAlive(_ghost, _ghost->object.gc);
      XSetFillStyle(TheDisplay, _ghost->object.gc, FillSolid);
      exposeAfraid(_ghost, _ghost->object.gc);
      break;
    case Blink:
      _ghost->ghost.state = !_ghost->ghost.state;
      if (_ghost->ghost.state)
        goto gotoAfraid;
      else
        goto gotoAlive;
      break;
  }

  SETCOLOR(_ghost->object.gc, White);
  exposeEyes(_ghost, _ghost->object.gc);

}

static void unDraw(GhostClass _ghost)
{
  switch(_ghost->object.state)
  {
    case Alive:
      exposeAlive(_ghost, TheBackgroundGC);
      break;
    case Afraid:
    case Blink:
      exposeAlive(_ghost, TheBackgroundGC);
      exposeAfraid(_ghost, TheBackgroundGC);
      break;
  } 
  exposeEyes(_ghost, TheBackgroundGC);
}

static void move(GhostClass _ghost)
{
  Direction dir;
  char *ptr;

  if (_ghost->object.direction == Still)
    return;

  ptr = OnTopOfChar((ObjectClass)_ghost);

  if (_ghost->object.state == Blink)
  {
    if (_ghost->object.clock <= 0)
    {
      ghost.ghost.numAfraid--;
      checkAfraid();
      _ghost->object.state = Alive;
      _ghost->object.clock = 0;
    }
    else
      _ghost->object.clock--;
  }
  else
  if (_ghost->object.state == Afraid)
  {
   if (_ghost->object.clock <= 25)
      _ghost->object.state = Blink;
   else
      _ghost->object.clock--;
  }


  if (*ptr == GHOST_CHAR)
  {
    if ((_ghost->object.state == Afraid) || (_ghost->object.state == Blink))
      ghost.ghost.numAfraid--;
    _ghost->object.state = Alive;
    _ghost->object.speed = _ghost->ghost.oldSpeed;
    _ghost->object.clock = 0;
  }

  switch(_ghost->object.state)
  {
    case Alive: 
      if ( (*ptr == PEN_CHAR) || (*ptr == GHOST_CHAR))
      {
        dir = MoveToChar(_ghost, DOT_CHAR);
        if (dir == Still)
          dir = MoveToChar(_ghost, PATH_CHAR);
      }
      else
        dir = MoveToKill(_ghost);
      break;
    case Eyes:
      dir = MoveTowardPen(_ghost);
      break;
    case Afraid:
    case Blink:
      dir = RunLikeHell(_ghost);
      break;
    default:
      dir = MoveToKill(_ghost);
      break;
  }

  switch(dir)
  {
    case Up:
      _ghost->object.y -= _ghost->object.moveInc;
      _ghost->object.x = ROUND(_ghost->object.x);
      break;
    case Down:
      _ghost->object.y += _ghost->object.moveInc;
      _ghost->object.x = ROUND(_ghost->object.x);
      break;
    case Left:
      _ghost->object.x -= _ghost->object.moveInc;
      _ghost->object.y = ROUND(_ghost->object.y);
      break;
    case Right:
      _ghost->object.x += _ghost->object.moveInc;
      _ghost->object.y = ROUND(_ghost->object.y);
      break;
    case Still:
      break;
  }

  _ghost->object.direction = dir;
}

static void classInit()
{
  Pixmap pix;
  char *stipple, *ptr, bit;
  unsigned long width, height;
  int i, j, k, mod;

  XQueryBestStipple(TheDisplay, TheWindow, TheBoxWidth, TheBoxHeight, &width, &height);

  stipple = (char *)Xmalloc(sizeof(char)*(width*height/8));

  ptr = stipple;
  *ptr = 0;
  bit = 0;
  for(j=0;j<height;j++)
  {
    bit = (bit ? 0 : 1);
    for(i=0;i<width;i++)
    {
      mod = (j*width+i) % 8;
      if (j && !mod)
      {
        ptr++;
        if (ptr - stipple)
           *ptr = 0;
      }
      *ptr |= (bit << mod);
      bit = (bit ? 0 : 1);
    }
  }
 
  pix = XCreateBitmapFromData(TheDisplay, TheWindow, stipple, width, height);

  XFree(stipple);
  XSetStipple(TheDisplay, ghost.object.gc, pix);

  ghost.object.color = White;
}

static int destroy(GhostClass _ghost)
{
  exposeAlive(_ghost, TheBackgroundGC);
  exposeAfraid(_ghost, TheBackgroundGC);
  DrawMazeCharUnderObject((ObjectClass)_ghost);
  _ghost->ghost.oldSpeed = _ghost->object.speed;
  _ghost->object.state = Eyes;
  _ghost->object.speed = Warp9;
  _ghost->ghost.value = ghost.ghost.value;
  ghost.ghost.numAfraid--;
  checkAfraid();
  return ghost.ghost.value;
}

static void checkAfraid()
{
   if (ghost.ghost.numAfraid == 0)
     ghost.ghost.value = 100 * (int)CEIL(TheLevel/2);
   else
     ghost.ghost.value *= 2;
} 


static void reset(GhostClass _ghost)
{
  char *ptr;

  DrawMazeCharUnderObject((ObjectClass)_ghost);
  ptr = strchr(TheMaze, GHOST_CHAR);
  if (ptr == (char *)NULL)
    ptr = strchr(TheMaze, PEN_CHAR);
 
  MoveObject((ObjectClass)_ghost, XPOS(ptr-TheMaze), YPOS(ptr-TheMaze));
  _ghost->object.state = Alive;
  _ghost->object.direction = Up;
  _ghost->object.clock = 0;
}

void ScareGhost(GhostClass _ghost)
{
  _ghost->object.unDraw(_ghost);
  if (Random() % 2)
    _ghost->object.direction = Reverse((ObjectClass)_ghost);
  _ghost->object.state = Afraid;
  _ghost->object.clock = ROUND(50.0/((double)TheLevel/3.0));
  ghost.ghost.numAfraid++;
}
