/*
** Generic list, stack, and queue handler.
** JLC
*/

#ifndef _LIST_H
#define _LIST_H 

#include <stdio.h>
#include <sys/types.h>

#define ERROR -1
#define OK 0

#define NOT_FOUND -10
#define NO_COMPARE_FUNC -11
#define LIST_HEAD_NULL -12
#define LIST_EMPTY -13
#define DATA_EMPTY -14 
#define NO_PRINT_FUNC -15

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif /* TRUE */

enum _Where
{
  Head = 1,
  Tail,
};

typedef struct _ListRec
{
  caddr_t data;
  struct _ListRec *next;
} ListRec, *ListPtr;

typedef struct _ListHead
{
  int length;
  ListPtr list;
  int (*compareFunc)();
#if DEBUGGING
  void (*printFunc)();
#endif /* DEBUGGING */
} ListHead, *ListHeadPtr;

typedef ListHeadPtr List;

#if NeedFunctionProtoTypes
extern List ListCreate( int (*compareFunc)(), void (*printFunc)());
extern int ListAdd(List *_l, caddr_t _data, int _where);
extern int ListRemove(List *_l, caddr_t _data);
extern int ListFind(List _l, caddr_t _data);
extern caddr_t ListFirst(List _l);
extern caddr_t ListLast(List _l);
extern int ListEmpty(List _l);
extern int ListLength(List _l);
extern void PrintList(List _l);
extern void ListDestroy(List *_l);
#else
extern List ListCreate();
extern int ListAdd();
extern int ListRemove();
extern int ListFind();
extern caddr_t ListFirst();
extern caddr_t ListLast();
extern int ListEmpty();
extern int ListLength();
extern void PrintList();
extern void ListDestroy();
#endif

#endif /* _LIST_H */
