#include "myIncludes.h"

#define MAZEDIR "mazes"
#define DEFAULTDIR "."

#if NeedFunctionProtoTypes
static void roundLeft(Drawable _d, int _col, int _row);
static void roundRight(Drawable _d, int _col, int _row);
static void roundUp(Drawable _d, int _col, int _row);
static void roundDown(Drawable _d, int _col, int _row);
static void wall(Drawable _d, int _col, int _row);
static void pen(Drawable _d, int _col, int _row);
static void dot(Drawable _d, int _col, int _row);
static void power(Drawable _d, int _col, int _row);
static void tunnel(Drawable _d, int _col, int _row);
#else
static void roundLeft();
static void roundRight();
static void roundUp();
static void roundDown();
static void wall();
static void pen();
static void dot();
static void power();
static void tunnel();
#endif

static int NumberOfDots = 0;

static int MazeFile = 1;

void ResetMazes()
{
  MazeFile = 1;
}

int ReadMazeFile()
{
  FILE *fid;
  char *oldMaze;
  char *dir;
  int mazeCount = 0;
  char *ptr;
  int i;
  char file[100];

  dir = (char *)getenv("XFREDDIR");

  if (dir == (char *)NULL)
    dir = DEFAULTDIR;

  sprintf(file, "%s/%s/maze.%d", dir, MAZEDIR, MazeFile);
  MazeFile++;


  if ((fid = fopen(file, "r")) == (FILE *)NULL)
  {
    MazeFile = 1;
    return FALSE;
  }

  oldMaze = TheMaze;
  TheMaze = (char *)Xmalloc(sizeof(char)*(MAZE_HEIGHT*MAZE_WIDTH+1));

  if (TheMaze == (char *)NULL)
  {
    TheMaze = oldMaze;
    fclose(fid);
    return;
  }

  ptr = TheMaze;
  for(i=0;i<MAZE_HEIGHT;i++)
  {
    mazeCount += fread((void *)ptr, sizeof(char), MAZE_WIDTH, fid);
    while (fgetc(fid) != '\n');
    ptr += MAZE_WIDTH;
  }

  if (mazeCount != (MAZE_HEIGHT * MAZE_WIDTH))
  {
    if (TheMaze)
      XFree(TheMaze);
    TheMaze = oldMaze;
    fclose(fid);
    return FALSE;
  }

  if (oldMaze)
    XFree(oldMaze);

  TheMaze[MAZE_HEIGHT*MAZE_WIDTH] = 0;
  fclose(fid);

  return TRUE;
}

void DrawMazeCharUnderObject(ObjectClass _object)
{
  char *ptr;
  int x[2], y[2], i;
  int j;

  if (TheMaze == (char *)NULL)
    return;

  switch(_object->object.direction)
  {
    case Still:
      x[0] = ROUND(_object->object.x);
      y[0] = ROUND(_object->object.y);
      j = 1;
      break;
    case Up:
    case Down:
      x[0] = x[1] = ROUND(_object->object.x);
      y[0] = FLOOR(_object->object.y);
      y[1] = CEIL(_object->object.y); 
      if (y[0] == y[1])
        j = 1;
      else
        j = 2;
      break;
    case Left:
    case Right:
      y[0] = y[1] = ROUND(_object->object.y); 
      x[0] = FLOOR(_object->object.x);
      x[1] = CEIL(_object->object.x); 
      if (x[0] == x[1])
        j = 1;
      else
        j = 2;
      break;
    default:
      printf("What direction is %d?\n", _object->object.direction);
  }

  for(i=0;i<j;i++)
  {
    ptr = TheMaze+(x[i]+y[i]*MAZE_WIDTH);
    switch(*ptr)
    {
      case EMPTY:
        break;
      case DOT_CHAR:
        dot(TheWindow, x[i], y[i]);
        break;
      case PEN_CHAR:
      case GHOST_CHAR:
        pen(TheWindow, x[i], y[i]); 
        break;
      case POWER_CHAR:
        power(TheWindow, x[i], y[i]);
        break;
      case TUNNEL_CHAR:
        tunnel(TheWindow, x[i], y[i]);
    }
  }
}

void DrawMaze()
{
  int i, j;
  char *ptr;

  XFillRectangle(TheDisplay, TheWindow, TheBackgroundGC, 
    0, 0, TheWidth, TheHeight);

  ptr = TheMaze;

  for(j=0; j < MAZE_HEIGHT; j++)
  {
    for(i=0; i < MAZE_WIDTH; i++)
    {
      switch(*ptr)
      {
        case ROUND_LEFT_CHAR:
          roundLeft(TheWindow, i, j);
          break;
        case ROUND_RIGHT_CHAR:
          roundRight(TheWindow, i, j);
          break;
        case WALL_CHAR:
          wall(TheWindow, i, j);
          break;
        case ROUND_UP_CHAR:
          roundUp(TheWindow, i, j);
          break;
        case ROUND_DOWN_CHAR:
          roundDown(TheWindow, i, j);
          break;
        case PEN_CHAR:
        case GHOST_CHAR:
          pen(TheWindow, i, j);
          break;
        case DOT_CHAR:
          dot(TheWindow, i, j);
          break;
        case POWER_CHAR:
          power(TheWindow, i, j);
          break;
        case TUNNEL_CHAR:
          tunnel(TheWindow, i, j);
      }
      ptr++;
    }
  }

  XFlush(TheDisplay);
}

void InitializeMaze()
{
  int i, j;
  char *ptr1, *ptr2;

/*
** Now fill in rest of maze with dots.
*/

  ptr1 = TheMaze;
  NumberOfDots = 0;

  for(j=0; j < MAZE_WIDTH; j++)
    for(i=0; i < MAZE_HEIGHT; i++)
    {
      if ((*ptr1 == PATH_CHAR))
      {
        *ptr1 = DOT_CHAR;
        NumberOfDots++;
      }
      ptr1++;
    }
}


static void roundLeft(Drawable _d, int _col, int _row)
{
  SETCOLOR(TheMazeGC, TheMazeColor);
  XFillRectangle(TheDisplay, _d, TheMazeGC, 
    XCOORD(_col)+TheBoxWidth/2, YCOORD(_row),
    TheBoxWidth/2+1, TheBoxHeight+1);

  XFillArc(TheDisplay, _d, TheMazeGC,
    XCOORD(_col)+1, YCOORD(_row), TheBoxWidth, TheBoxHeight,
    90*64, 270*64);
} 

static void roundRight(Drawable _d, int _col, int _row)
{
  SETCOLOR(TheMazeGC, TheMazeColor);
  XFillRectangle(TheDisplay, _d, TheMazeGC,
    XCOORD(_col), YCOORD(_row), 
    TheBoxWidth/2+1, TheBoxHeight+1);

  XFillArc(TheDisplay, _d, TheMazeGC,
    XCOORD(_col), YCOORD(_row), TheBoxWidth, TheBoxHeight,
    0, 360*64);
}

static void roundUp(Drawable _d, int _col, int _row)
{
  SETCOLOR(TheMazeGC, TheMazeColor);
  XFillRectangle(TheDisplay, _d, TheMazeGC,
    XCOORD(_col), YCOORD(_row)+TheBoxHeight/2,
    TheBoxWidth+1, TheBoxHeight/2+1);

  XFillArc(TheDisplay, _d, TheMazeGC,
    XCOORD(_col), YCOORD(_row), 
    TheBoxWidth, TheBoxHeight,
    0*64, 180*64);
}

static void roundDown(Drawable _d, int _col, int _row)
{
  SETCOLOR(TheMazeGC, TheMazeColor);
  XFillRectangle(TheDisplay, _d, TheMazeGC,
    XCOORD(_col), YCOORD(_row),
    TheBoxWidth+1, TheBoxHeight/2+1);

  XFillArc(TheDisplay, _d, TheMazeGC,
    XCOORD(_col), YCOORD(_row),
    TheBoxWidth, TheBoxHeight,
    180*64, 360*64);
}

static void wall(Drawable _d, int _col, int _row)
{
  SETCOLOR(TheMazeGC, TheMazeColor);
  XFillRectangle(TheDisplay, _d, TheMazeGC,
    XCOORD(_col), YCOORD(_row),
    TheBoxWidth+1, TheBoxHeight+1);
}

static void tunnel(Drawable _d, int _col, int _row)
{
  XSegment line[4];
  line[0].x1 = XCOORD(_col);
  line[0].y1 = line[1].y1 = line[2].y1 = line[3].y1 = YCOORD(_row);
  line[1].x1 = line[0].x1 + TheBoxWidth/4;
  line[2].x1 = line[1].x1 + TheBoxWidth/4;
  line[3].x1 = line[2].x1 + TheBoxWidth/4;
  line[0].x2 = line[0].x1;
  line[1].x2 = line[1].x1;
  line[2].x2 = line[2].x1;
  line[3].x2 = line[3].x1;
  line[0].y2 = line[1].y2 = line[2].y2 = line[3].y2 = line[0].y1 + TheBoxHeight;
  SETCOLOR(TheMazeGC, TheTunnelColor);
  XDrawSegments(TheDisplay, TheWindow, TheMazeGC, line, 4);
}

static void pen(Drawable _d, int _col, int _row)
{
  SETCOLOR(TheMazeGC, ThePenColor);
  XFillRectangle(TheDisplay, _d, TheMazeGC,
    XCOORD(_col), YCOORD(_row),
    TheBoxWidth+1, TheBoxHeight+1);
} 

static void dot(Drawable _d, int _col, int _row)
{
  SETCOLOR(TheMazeGC, TheDotColor);
  XFillArc(TheDisplay, _d, TheMazeGC, 
    XCOORD(_col)+TheBoxWidth/4, YCOORD(_row)+TheBoxHeight/4,
    TheBoxWidth/2, TheBoxHeight/2,
    0, 360*64);
}

static void power(Drawable _d, int _col, int _row)
{
  SETCOLOR(TheMazeGC, ThePowerColor);
  XFillRectangle(TheDisplay, TheWindow, TheMazeGC, XCOORD(_col)+4, YCOORD(_row)+4, TheBoxWidth - 8,
    TheBoxHeight-8);
}

char *OnTopOfChar(ObjectClass _object)
{
  int x, y;

  x = ROUND(_object->object.x);
  y = ROUND(_object->object.y);

  return (char *)(TheMaze+x+y*MAZE_WIDTH);
}

int GoodUp(ObjectClass _object)
{
  int test;
  int x, y;

  if (!SOLID(_object->object.x))
    return FALSE;

  x = ROUND(_object->object.x);
  y = CEIL(_object->object.y);

  if (y == 0)
    return FALSE;

  test = MAZE_WIDTH * (y - 1) + x;

  switch(TheMaze[test])
  {
    case PATH_CHAR:
    case FRUIT_CHAR:
    case TUNNEL_CHAR:
    case DOT_CHAR:
    case POWER_CHAR:
    case FRED_CHAR:
      return TRUE;
      break;
    case PEN_CHAR:
    case GHOST_CHAR:
      if (_object->object.type == GhostObject)
      {
        char *ptr;
        ptr = OnTopOfChar(_object);
        if ((*ptr == PEN_CHAR) || (*ptr == GHOST_CHAR))
          return TRUE;
        else
        if (((GhostClass)_object)->object.state == Eyes)
          return TRUE;
      }
      return FALSE;
    default:
      return FALSE;
      break;
  }
}

int GoodDown(ObjectClass _object)
{
  int test;
  int x, y;

  if (!SOLID(_object->object.x))
    return FALSE;

  x = ROUND(_object->object.x);
  y = FLOOR(_object->object.y);

  if (y == (MAZE_HEIGHT - 1))
    return FALSE;

  test = MAZE_WIDTH * (y + 1) + x;

  switch(TheMaze[test])
  {
    case PATH_CHAR:
    case FRUIT_CHAR:
    case TUNNEL_CHAR:
    case DOT_CHAR:
    case POWER_CHAR:
    case FRED_CHAR:
      return TRUE;
      break;
    case PEN_CHAR:
    case GHOST_CHAR:
      if (_object->object.type == GhostObject)
      {
        char *ptr;
        ptr = OnTopOfChar(_object);
        if ((*ptr == PEN_CHAR) || (*ptr == GHOST_CHAR))
          return TRUE;
        else
        if (((GhostClass)_object)->object.state == Eyes)
          return TRUE;
      }
      return FALSE;
    default:
      return FALSE;
      break;
  }
}

int GoodLeft(ObjectClass _object)
{
  int test;
  int x, y;

  if (!SOLID(_object->object.y))
    return FALSE;

  x = CEIL(_object->object.x);
  y = ROUND(_object->object.y);

  if (x == 0)
    return FALSE;

  test =  MAZE_WIDTH * y + x - 1;

  switch(TheMaze[test])
  {
    case PATH_CHAR:
    case FRUIT_CHAR:
    case TUNNEL_CHAR:
    case DOT_CHAR:
    case POWER_CHAR:
    case FRED_CHAR:
      return TRUE;
      break;
    case PEN_CHAR:
    case GHOST_CHAR:
      if (_object->object.type == GhostObject)
      {
        char *ptr;
        ptr = OnTopOfChar(_object);
        if ((*ptr == PEN_CHAR) || (*ptr == GHOST_CHAR))
          return TRUE;
        else
        if (((GhostClass)_object)->object.state == Eyes)
          return TRUE;
      }
      return FALSE;
    default:
      return FALSE;
      break;
  }
}

int GoodRight(ObjectClass _object)
{
  int test;
  int x, y;

  if (!SOLID(_object->object.y))
    return FALSE;

  x = FLOOR(_object->object.x);
  y = ROUND(_object->object.y);
 
  if (x == (MAZE_WIDTH - 1))
    return FALSE;
 
  test =  MAZE_WIDTH * y + x + 1;
 
  switch(TheMaze[test])
  {
    case PATH_CHAR:
    case FRUIT_CHAR:
    case TUNNEL_CHAR:
    case DOT_CHAR:
    case POWER_CHAR:
    case FRED_CHAR:
      return TRUE;
      break;
    case PEN_CHAR:
    case GHOST_CHAR:
      if (_object->object.type == GhostObject)
      {
        char *ptr;
        ptr = OnTopOfChar(_object);
        if ((*ptr == PEN_CHAR) || (*ptr == GHOST_CHAR))
          return TRUE;
        else
        if (((GhostClass)_object)->object.state == Eyes)
          return TRUE;
      }
      return FALSE;
    default:
      return FALSE;
      break;
  }
}

int GoodDir(ObjectClass _object, Direction _dir)
{
  switch(_dir)
  {
    case Up:
      return GoodUp(_object);
    case Down:
      return GoodDown(_object);
    case Left:
      return GoodLeft(_object);
    case Right:
      return GoodRight(_object);
  }
}

int RemoveMazeElement(double _dx, double _dy)
{
  int x, y;
  char val;
  char *ptr;

  if (!SOLID(_dx) || !SOLID(_dy))
    return 0;

  x = ROUND(_dx);
  y = ROUND(_dy);

  ptr = TheMaze+x+y*MAZE_WIDTH;

  if (*ptr == EMPTY)
    return EMPTY;

  switch (*ptr)
  {
    case POWER_CHAR:
      ((FredClass)TheFred)->fred.score += 100;
      break;
    case DOT_CHAR:
      ((FredClass)TheFred)->fred.score += 10;
      NumberOfDots--;
      break;
    default:
      return EMPTY;
  }

  if (NumberOfDots == 0)
  {
    ((FredClass)TheFred)->fred.score += 500*TheLevel/2;
    NewLevel();
    GameClock = 0;
    DrawMaze();
  }

  val = *ptr;
  *ptr = EMPTY;

  return val;
}
  

 


  

