#include "myIncludes.h"

static Direction getBestDir(
#if NeedFunctionProtoTypes
ObjectClass _object, double _x, double _y
#endif
);

void TunnelWarp(ObjectClass _object)
{
  char *ptr, *next;
  int x, y;

  if (!SOLID(_object->object.x) || !SOLID(_object->object.y))
    return;

  x = ROUND(_object->object.x);
  y = ROUND(_object->object.y);

  ptr = TheMaze+x+y*MAZE_WIDTH;

  if (*ptr != TUNNEL_CHAR)
    return;

  next = strchr(ptr+1, TUNNEL_CHAR);

  if (next == (char *)NULL)
    next = strchr(TheMaze, TUNNEL_CHAR);

  _object->object.x = XPOS(next-TheMaze);
  _object->object.y = YPOS(next-TheMaze);
}

/*
** Move in a random direction at any intersection, plus make
** sure that you don't reverse direction unless totally necessary.
*/

Direction MoveRandom(ObjectClass _object)
{
  int number;

  if (!SOLID(_object->object.x) || !SOLID(_object->object.y))
    return _object->object.direction;

  if (_object->object.direction == Still)
    return Still;

  number = Random() % 3; 

  switch(_object->object.direction)
  {
    case Up:
      switch(number)
      {
        case 0:
          if (GoodUp(_object))
            return Up;
          else
          if (GoodLeft(_object))
            return Left;
          else
          if (GoodRight(_object))
            return Right;
          else
            return Down;
        case 1:
          if (GoodLeft(_object))
            return Left;
          else
          if (GoodRight(_object))
            return Right;
          else
          if (GoodUp(_object))
            return Up;
          else
            return Down;
        case 2:
          if (GoodRight(_object))
            return Right;
          else
          if (GoodUp(_object))
            return Up;
          else
          if (GoodLeft(_object))
            return Left;
          else
            return Down;
      }
    case Down:
      switch(number % 3)
      {
        case 0:
          if (GoodDown(_object))
            return Down;
          else
          if (GoodLeft(_object))
            return Left;
          else
          if (GoodRight(_object))
            return Right;
          else
            return Up;
        case 1:
          if (GoodLeft(_object))
            return Left;
          else
          if (GoodRight(_object))
            return Right;
          else
          if (GoodDown(_object))
            return Down;
          else
            return Up;
        case 2:
          if (GoodRight(_object))
            return Right;
          else
          if (GoodDown(_object))
            return Down;
          else
          if (GoodLeft(_object))
            return Left;
          else
            return Up;
      }
    case Left:
      switch(number % 3)
      {
        case 0:
          if (GoodLeft(_object))
            return Left;
          else
          if (GoodDown(_object))
            return Down;
          else
          if (GoodUp(_object))
            return Up;
          else
            return Right;
        case 1:
          if (GoodUp(_object))
            return Up;
          else
          if (GoodDown(_object))
            return Down;
          else
          if (GoodLeft(_object))
            return Left;
          else
            return Right;
        case 2:
          if (GoodDown(_object))
            return Down;
          else
          if (GoodUp(_object))
            return Up;
          else
          if (GoodLeft(_object))
            return Left;
          else
            return Right;
      }
    case Right:
      switch(number % 3)
      {
        case 0:
          if (GoodRight(_object))
            return Right;
          else
          if (GoodDown(_object))
            return Down;
          else
          if (GoodUp(_object))
            return Up;
          else
            return Left;
        case 1:
          if (GoodUp(_object))
            return Up;
          else
          if (GoodRight(_object))
            return Right;
          else
          if (GoodDown(_object))
            return Down;
          else
            return Left; 
        case 2:
          if (GoodDown(_object))
            return Down;
          else
          if (GoodUp(_object))
            return Up;
          else
          if (GoodRight(_object))
            return Right;
          else
            return Left;
      }
  }
}

/*
** Make a list of the directions in order of the worst to best directions
** to take.  Then index into that with the intelligence of the ghost.
** Neato.
*/

/*
** uD negative means  that fred is below 
** uL negative means  that fred is right
*/

#define INFINITY (MAZE_WIDTH * MAZE_HEIGHT * 2.0)

extern double sqrt();

#define SQR(a) ((double)((a)*(a)))
#define OBJDIST(dx,dy,toX, toY) (sqrt( SQR((dx)-toX) + SQR((dy)-toY) ) )

static Direction getBestDir(ObjectClass _object, double _x, double _y)
{
  double distance[3];
  Direction dirList[3];
  Direction yuccDir;
  ObjectClass object;
  int count = 0;

  if (_object->object.direction == Still)
    return Still;

  switch(_object->object.direction)
  {
    case Up:
      yuccDir = Down;
      if (GoodUp(_object))
      {
        distance[count] = OBJDIST(_object->object.x, _object->object.y-1, _x, _y);
        dirList[count] = Up;
        count++;
      }
      if (GoodRight(_object))
      {
        distance[count] = OBJDIST(_object->object.x+1, _object->object.y, _x, _y);
        dirList[count] = Right;
        count++;
      }
      if (GoodLeft(_object))
      {
        distance[count] = OBJDIST(_object->object.x-1, _object->object.y, _x, _y);
        dirList[count] = Left;
        count++;
      }
      break;
    case Down:
      yuccDir = Up;
      if (GoodDown(_object))
      {
        distance[count] = OBJDIST(_object->object.x, _object->object.y+1, _x, _y);
        dirList[count] = Down;
        count++;
      }
      if (GoodLeft(_object))
      {
        distance[count] = OBJDIST(_object->object.x-1, _object->object.y, _x, _y);
        dirList[count] = Left;
        count++;
      }
      if (GoodRight(_object))
      {
        distance[count] = OBJDIST(_object->object.x+1, _object->object.y, _x, _y);
        dirList[count] = Right;
        count++;
      }
      break;
    case Left:
      yuccDir = Right;
      if (GoodUp(_object))
      {
        distance[count] = OBJDIST(_object->object.x, _object->object.y-1, _x, _y);
        dirList[count] = Up;
        count++;
      }
      if (GoodLeft(_object))
      {
        distance[count] = OBJDIST(_object->object.x-1, _object->object.y, _x, _y);
        dirList[count] = Left;
        count++;
      }
      if (GoodDown(_object))
      {
        distance[count] = OBJDIST(_object->object.x, _object->object.y+1, _x, _y);
        dirList[count] = Down; 
        count++;
      }
      break;
    case Right:
      yuccDir = Left;
      if (GoodDown(_object))
      {
        distance[count] = OBJDIST(_object->object.x, _object->object.y-1, _x, _y);
        dirList[count] = Down;
        count++;
      }
      if (GoodRight(_object))
      {
        distance[count] = OBJDIST(_object->object.x+1, _object->object.y, _x, _y);
        dirList[count] = Right;
        count++;
      }
      if (GoodUp(_object))
      {
        distance[count] = OBJDIST(_object->object.x, _object->object.y+1, _x, _y);
        dirList[count] = Up;
        count++;
      }
      break;
  }
 
  if (count == 0)
    return yuccDir;

  if (count == 1)
    return dirList[0];

  if (count == 2)
    if (distance[0] < distance[1])
      return dirList[0];
    else
      return dirList[1];

  if (count == 3)
  {
    Direction bestDir;
    int bestDistance;
    int i;

    bestDir = dirList[0];
    bestDistance = distance[0];

    for(i=1;i<2;i++)
      if (bestDistance > distance[i])
      {
        bestDir = dirList[i];
        bestDistance = distance[i];
      } 

    return bestDir;
  }

}

Direction MoveToKill(GhostClass _ghost)
{
  Direction dir, goodDir;
  ObjectClass object;

  object = (ObjectClass)_ghost;

  if (!SOLID(object->object.x) || !SOLID(object->object.y))
    return object->object.direction;

  if ( Random() <= _ghost->ghost.iQ)
    return MoveRandom(object);
 
  return getBestDir((ObjectClass)_ghost, TheFred->object.x, TheFred->object.y);
}

Direction RunLikeHell(GhostClass _ghost)
{
  ObjectClass object = (ObjectClass)_ghost;
  int x, y;

  if (!SOLID(object->object.x) || !SOLID(object->object.y))
    return object->object.direction;

  if ( Random() <= _ghost->ghost.iQ)
    return MoveRandom(object);

  x = MAZE_WIDTH - FLOOR(object->object.x);
  y = MAZE_HEIGHT - FLOOR(object->object.y);

  return getBestDir(object, x, y);
}

Direction MoveToChar(ObjectClass _object, char _toChar)
{
  char *ptr;
  int x, y;
 
  if (!SOLID(_object->object.x) || !SOLID(_object->object.y))
    return _object->object.direction;

  ptr = strchr(TheMaze, _toChar);

  if (ptr == (char*)NULL)
    return Still;

  x = XPOS(ptr-TheMaze);
  y = YPOS(ptr-TheMaze);

  return getBestDir(_object, x, y);
}

Direction MoveTowardPen(GhostClass _ghost)
{
  ObjectClass object;
  char *ptr, *onChar;
  int x, y;

  object = (ObjectClass)_ghost;
 
  if (!SOLID(object->object.x) || !SOLID(object->object.y))
    return object->object.direction;
 
  ptr = strchr(TheMaze, GHOST_CHAR);
  if (ptr == (char *)NULL)
    ptr = strchr(TheMaze, PEN_CHAR);

  onChar = OnTopOfChar((ObjectClass)_ghost);

  x = XPOS(ptr-TheMaze);
  y = YPOS(ptr-TheMaze);

  if (*onChar != PEN_CHAR)
  {
    if ((Random() % 4) == 0)
      return MoveRandom((ObjectClass)_ghost);
    else
      if (GoodUp((ObjectClass)_ghost))
      {
        ptr = TheMaze+ROUND(_ghost->object.x+(_ghost->object.y-1)*MAZE_WIDTH);
        if (*ptr == PEN_CHAR)
          return Up;
      }

      if (GoodDown((ObjectClass)_ghost))
      {
        ptr = TheMaze+ROUND(_ghost->object.x+(_ghost->object.y+1)*MAZE_WIDTH);
        if (*ptr == PEN_CHAR)
          return Down;
      }

      if (GoodRight((ObjectClass)_ghost))
      {
        ptr = TheMaze+ROUND(_ghost->object.x+1+_ghost->object.y*MAZE_WIDTH);
        if (*ptr == PEN_CHAR)
          return Right;
      }

      if (GoodLeft((ObjectClass)_ghost))
      {
        ptr = TheMaze+ROUND(_ghost->object.x-1+_ghost->object.y*MAZE_WIDTH);
        if (*ptr == PEN_CHAR)
          return Left;
      }
  }


  return getBestDir((ObjectClass)_ghost, x, y);
}

Direction Reverse(ObjectClass _object)
{
  switch(_object->object.direction)
  {
    case Up:
      if (GoodDown((ObjectClass)_object))
        return Down;
      break;
    case Down:
      if (GoodUp((ObjectClass)_object))
        return Up;
      break;
    case Left:
     if (GoodRight((ObjectClass)_object))
        return Right;
      break;
    case Right:
      if (GoodLeft((ObjectClass)_object))
        return Left;
      break;
    default:
      break;
  }
  return _object->object.direction;
}
