#include "myIncludes.h"

#define WIDTH 30
#define HEIGHT 60
#define SPACE 10

#define TOP_LED (1 << 0)
#define MIDDLE_LED (1 << 1)
#define BOTTOM_LED (1 << 2)
#define LT_LED (1 << 3)
#define RT_LED (1 << 4)
#define LB_LED (1 << 5)
#define RB_LED (1 << 6)

#define ZERO (TOP_LED | BOTTOM_LED | LT_LED | LB_LED | RT_LED | RB_LED)
#define ONE (RT_LED | RB_LED)
#define TWO (TOP_LED | RT_LED | MIDDLE_LED | LB_LED | BOTTOM_LED)
#define THREE (TOP_LED | MIDDLE_LED | BOTTOM_LED | RT_LED | RB_LED)
#define FOUR (LT_LED | MIDDLE_LED | RT_LED | RB_LED)
#define FIVE (TOP_LED | LT_LED | MIDDLE_LED | RB_LED | BOTTOM_LED)
#define SIX (TOP_LED | LT_LED | MIDDLE_LED | RB_LED | BOTTOM_LED | LB_LED)
#define SEVEN (TOP_LED | RT_LED | RB_LED)
#define EIGHT (TOP_LED | MIDDLE_LED | BOTTOM_LED | LT_LED | RT_LED | LB_LED | RB_LED)
#define NINE (TOP_LED | RT_LED | RB_LED | LT_LED | MIDDLE_LED)

#define NUM_DIGITS 6

static void exposeLed(Drawable _drawable, GC _theGC, unsigned int _x, unsigned int _y, unsigned int _leds)
{
  XSegment segs[7];
  int n;

  n = 0;

  if (_leds & TOP_LED)
  {
    segs[n].x1 = _x;
    segs[n].x2 = segs[n].x1 + WIDTH;
    segs[n].y1 = _y;
    segs[n].y2 = _y;
    n++;
  }

  if (_leds & MIDDLE_LED)
  {
    segs[n].x1 = _x; 
    segs[n].x2 = segs[n].x1 + WIDTH;
    segs[n].y1 = _y + HEIGHT/2;
    segs[n].y2 = segs[n].y1;
    n++;
  }
 
  if (_leds & BOTTOM_LED)
  {
    segs[n].x1 = _x; 
    segs[n].x2 = segs[n].x1 + WIDTH;
    segs[n].y1 = _y + HEIGHT;
    segs[n].y2 = segs[n].y1;
    n++;
  }
 
  if (_leds & LT_LED)
  {
    segs[n].x1 = _x; 
    segs[n].x2 = _x;
    segs[n].y1 = _y;
    segs[n].y2 = _y + HEIGHT/2;
    n++;
  }
 
  if (_leds & LB_LED)
  {
    segs[n].x1 = _x; 
    segs[n].x2 = _x;
    segs[n].y1 = _y + HEIGHT/2;
    segs[n].y2 = _y + HEIGHT;
    n++;
  }
 
  if (_leds & RT_LED)
  {
    segs[n].x1 = _x + WIDTH; 
    segs[n].x2 = segs[n].x1;
    segs[n].y1 = _y;
    segs[n].y2 = _y + HEIGHT/2;
    n++;
  }

  if (_leds & RB_LED)
  {
    segs[n].x1 = _x + WIDTH;
    segs[n].x2 = segs[n].x1;
    segs[n].y1 = _y + HEIGHT/2;
    segs[n].y2 = _y + HEIGHT;
    n++;
  }
 
  XDrawSegments(TheDisplay, _drawable, _theGC, segs, n);
}

void DrawScore(GC _theGC, int _score)
{
  unsigned int digit[NUM_DIGITS];
  int bit;
  int temp = _score;
  int i, x, y;

  for(i=0;i<NUM_DIGITS;i++)
  {
    bit = temp % 10;
    switch(bit)
    {
      case 0:
        digit[i] = ZERO;
        break;
      case 1:
        digit[i] = ONE;
        break;
      case 2:
        digit[i] = TWO;
        break;
      case 3:
        digit[i] = THREE;
        break;
      case 4:
        digit[i] = FOUR;
        break;
      case 5:
        digit[i] = FIVE;
        break;
      case 6:
        digit[i] = SIX;
        break;
      case 7:
        digit[i] = SEVEN;
        break;
      case 8:
        digit[i] = EIGHT;
        break;
      case 9:
        digit[i] = NINE;
        break;
    }
    temp /= 10;
  }
    
  x = STAT_WIN_WIDTH/2 - (NUM_DIGITS * WIDTH + (NUM_DIGITS-1) * SPACE)/2;
  y = STAT_WIN_WIDTH/4;
  for(i=NUM_DIGITS-1;i>=0;i--)
  {
    exposeLed(TheStatusWindow, _theGC, x, y, digit[i]);
    x += WIDTH + SPACE;
  }

}
